package com.luojigou.xiaodou.ble

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.bluetooth.ScanFilter
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.luojigou.xiaodou.R
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

@SuppressLint("MissingPermission")
class XDScanBLENormalFragment(private val host: XDScanBLEStatus.Host) : Fragment(),
    CoroutineScope by MainScope() {
    private val adapter = XDScanBLEAdapter {

    }

    private val scanFilter = listOf<ScanFilter>(
    )

    private lateinit var selectWifi: ActivityResultLauncher<Intent>

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_scanble_normal, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        selectWifi = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            host.onStatusChanged(XDScanBLEStatus.Disabled)
        }

        val recyclerView = view.findViewById<RecyclerView>(R.id.scan_list)
        val layoutManager = GridLayoutManager(requireContext(), 2, RecyclerView.VERTICAL, false)

        recyclerView.layoutManager = layoutManager
        recyclerView.adapter = adapter
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)

        launch {
            host.getBluetooth().scan(scanFilter).collect {
                val device = it.device
                val rssi = it.rssi
                val serviceUuids = it.serviceUuids
                val deviceAddress = it.deviceAddress
                val periodicAdvertisingInterval = it.periodicAdvertisingInterval
                val timestampNanos = it.timestampNanos
                val serviceData = it.serviceData

                val id = device.id
                val bondState = device.bondState
                val name = device.name

                val address = deviceAddress.address
                val addressType = deviceAddress.addressType

                Log.d("XDScanBLENormalFragment", "onAttach: ")

                if (name != null) {
                    withContext(Dispatchers.Main) {
                        adapter.addValue(
                            XDScanBLEAdapter.Item(
                                id = id.toString(),
                                name = name.toString(),
                                bondState = bondState.toString(),
                                rssi = rssi.toString(),
                                address = address,
                                addressType = addressType.toString()
                            )
                        )
                    }
                }
            }
        }
    }

    override fun onDetach() {
        adapter.setValues(mutableListOf())
        super.onDetach()
    }
}