package com.luojigou.xiaodou.wifi

import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.Fragment
import com.luojigou.xiaodou.R

class XDScanWifiPermissionFragment(private val host: XDScanWifiStatus.Host) : Fragment() {
    private val requestPermission: ActivityResultLauncher<Array<String>> =
        registerForActivityResult(ActivityResultContracts.RequestMultiplePermissions()) {
            if (it.all { permission -> permission.value }) {
                host.onStatusChanged(XDScanWifiStatus.Normal)
            } else {
                Log.e("XDScanWifiActivity", "requestPermission: 权限被拒绝")
            }
        }

    private val permissions by lazy {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            arrayOf(
                android.Manifest.permission.ACCESS_FINE_LOCATION,
                android.Manifest.permission.CHANGE_WIFI_STATE
            )
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            arrayOf(android.Manifest.permission.CHANGE_WIFI_STATE)
        } else {
            arrayOf()
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View {
        return inflater.inflate(R.layout.fragment_scanwifi_permission, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val button = view.findViewById<Button>(R.id.btn_open)
        button.setOnClickListener { requestPermission() }
    }

    private fun requestPermission() {
        requestPermission.launch(permissions)
    }
}